---
elm:
  dependencies:
    avh4/elm-fifo: latest
    elm/browser: latest
    elm/core: latest
    elm/html: latest
    elm/json: latest
    elm/parser: latest
    elm/regex: latest
    elm/time: latest
    elm/url: latest
    elm/virtual-dom: latest
    elm-community/graph: latest
    elm-community/intdict: latest
    elm-community/list-extra: latest
    elm-community/maybe-extra: latest
    elm-community/string-extra: latest

  source-directories:
    - ../src
---

# El problema del coloreado del grafos

> Fuente: Ejercios de Lógica Computacional
> (https://www.cs.us.es/~fsancho/?p=logica-informatica-2020-21)



Es bien sabido que todo grafo camino es un grafo bipartito, esto es sus vértices se pueden organizar en dos conjuntos de forma que no existan aristas entre vértices de un mismo conjunto. También se sabe que los ciclos de un número impar de vértices no lo son. Se pide, haciendo uso de DPLL, comprobar estas propiedades para el grafo camino $P_3$ y el grafo ciclo $C_3$.

<div style="width:50%; float:left">
<center>

```dot
digraph G {
    rankdir = LR
    node[shape=circle]

    p₁ -> p₂ -> p₃
    
}
```
</center>
</div>
<div style="width:50%; float:left">
<center>

```dot
digraph G {
    rankdir = LR
    node[shape=circle]

    c₁ -> c₂ -> c₃ -> c₁
    
}
```
</center>
</div>

## Solución

Empecemos probando que el grafo $P_3$ es bipartito. Para ello sean las variables proposicionales:

- $a_i$: el vértice $p_i$ pertenece al conjunto $A$

- $b_i$: el vértice $b_i$ pertenece al conjunto $B$

Así podemos formalizar el enunciado en:

- Todo vértice pertenece a uno de los conjuntos: $(a_1 \vee b_1) \wedge (a_2 \vee b_2) \wedge (a_3 \vee b_3) $

- Dos vértices adyacentes no pueden pertenecer (ambos) al conjunto $A$ : $\neg (a_1 \wedge a_2) \wedge \neg (a_2 \wedge a_3)$ 

- Dos vértices adyacentes no pueden pertenecer (ambos) al conjunto $B$ : $\neg (b_1 \wedge b_2) \wedge \neg (b_2 \wedge b_3)$ 

Vamos a demostrar la inconsistencia del conjunto formado por las fórmulas anteriores utilizando LogicUS.

```elm {l id="imports"}
import LogicUS.PL.SintaxSemantics exposing (..)
import LogicUS.PL.Clauses exposing (..)
import LogicUS.PL.DPLL exposing (..)

fplRead : String -> FormulaPL
fplRead = fplReadExtraction << fplReadFromString
```

Definimos las fórmulas:

```elm {l context="1" follows="imports"}
f1 : FormulaPL
f1 = fplRead "(a_{1} | b_{1}) & (a_{2} | b_{2}) & (a_{3} | b_{3})"

f2 : FormulaPL
f2 = fplRead "¬(a_{1} & a_{2}) & ¬(a_{2} & a_{3})"

f3 : FormulaPL
f3 = fplRead "¬(b_{1} & b_{2}) & ¬(b_{2} & b_{3})"

```

De forma que le corresponde el conjunto de cláusulas:

```elm {l m=("$ " ++ (csplToMathString cs) ++ " $") context="1"}
cs : ClausePLSet
cs = splToClauses [f1,f2,f3]
```

<br> Que corresponde a una lista de cláusulas correspondientes a listas de literales expresados como pares _(símbolo,signo)_. Representándolas:

Ahora, aplicando el algoritmo DPLL y representándolo en formato DOT

```elm {l r context="1"}
csTableau : String
csTableau =  dpll cs |> dpllTableauToDOT
```


<br>Representándolo:

<center>

```dot
digraph G {
 rankdir=TB
 graph []
 node [shape=box, color=white, fontcolor=black]
 edge [dir=none, color=blue, fontcolor=blue]

 0 -> 1 [label=<a₂>]
 0 -> 7 [label=<¬ a₂>]
 1 -> 2 [label=<¬ a₁>]
 2 -> 3 [label=<b₁>]
 3 -> 4 [label=<¬ a₃>]
 4 -> 5 [label=<b₃>]
 5 -> 6 [label=<¬ b₂>]
 7 -> 8 [label=<b₂>]
 8 -> 9 [label=<¬ b₁>]
 9 -> 10 [label=<a₁>]
 10 -> 11 [label=<¬ b₃>]
 11 -> 12 [label=<a₃>]

 0 [label=<{a₁,b₁}, {a₂,b₂}, {a₃,b₃}, {¬ a₁,¬ a₂}, {¬ a₂,¬ a₃}, {¬ b₁,¬ b₂}, {¬ b₂,¬ b₃}>]
 1 [label=<{a₁,b₁}, {a₃,b₃}, {¬ a₁}, {¬ a₃}, {¬ b₁,¬ b₂}, {¬ b₂,¬ b₃}>]
 2 [label=<{b₁}, {a₃,b₃}, {¬ a₃}, {¬ b₁,¬ b₂}, {¬ b₂,¬ b₃}>]
 3 [label=<{a₃,b₃}, {¬ a₃}, {¬ b₂}>]
 4 [label=<{b₃}, {¬ b₂}>]
 5 [label=<{¬ b₂}>]
 6 [label=<◯>]
 7 [label=<{a₁,b₁}, {b₂}, {a₃,b₃}, {¬ b₁,¬ b₂}, {¬ b₂,¬ b₃}>]
 8 [label=<{a₁,b₁}, {a₃,b₃}, {¬ b₁}, {¬ b₃}>]
 9 [label=<{a₁}, {a₃,b₃}, {¬ b₃}>]
 10 [label=<{a₃,b₃}, {¬ b₃}>]
 11 [label=<{a₃}>]
 12 [label=<◯>]
}
```

</center>

Hagámos lo mismo para el grafo $C_3$. Para ello sean las variables proposicionales:

- $a_i$: el vértice $c_i$ pertenece al conjunto $A$

- $b_i$: el vértice $c_i$ pertenece al conjunto $B$

Así podemos formalizar el enunciado en:

- Todo vértice pertenece a uno de los conjuntos: $(a_1 \vee b_1) \wedge (a_2 \vee b_2) \wedge (a_3 \vee b_3) $

- Dos vértices adyacentes no pueden pertenecer (ambos) al conjunto $A$ : $\neg (a_1 \wedge a_2) \wedge \neg (a_2 \wedge a_3) \wedge \neg (a_3 \wedge a_1)$ 

- Dos vértices adyacentes no pueden pertenecer (ambos) al conjunto $B$ : $\neg (b_1 \wedge b_2) \wedge \neg (b_2 \wedge b_3) \wedge \neg (b_3 \wedge b_1)$ 

Vamos a demostrar la inconsistencia del conjunto formado por las fórmulas anteriores utilizando LogicUS.

Definimos las fórmulas:

```elm {l context="2" follows="imports"}
f1 : FormulaPL
f1 = fplRead "(a_{1} | b_{1}) & (a_{2} | b_{2}) & (a_{3} | b_{3})"

f2 : FormulaPL
f2 = fplRead "¬(a_{1} & a_{2}) & ¬(a_{2} & a_{3}) & ¬(a_{3} & a_{1})"

f3 : FormulaPL
f3 = fplRead "¬(b_{1} & b_{2}) & ¬(b_{2} & b_{3}) & ¬(b_{3} & b_{1})"

```

De forma que le corresponde el conjunto de cláusulas:

```elm {l m=("$ " ++ (csplToMathString cs) ++ " $") context="2"}
cs : ClausePLSet
cs = splToClauses [f1,f2,f3]
```

<br> Que corresponde a una lista de cláusulas correspondientes a listas de literales expresados como pares _(símbolo,signo)_. Representándolas:

Ahora, aplicando el algoritmo DPLL y representándolo en formato DOT

```elm {l r context="2"}
csTableau : String
csTableau =  dpll cs |> dpllTableauToDOT
```


<br>Representándolo:

<center>

```dot
digraph G {
 rankdir=TB
 graph []
 node [shape=box, color=white, fontcolor=black]
 edge [dir=none, color=blue, fontcolor=blue]

 0 -> 1 [label=<a₁>]
 0 -> 6 [label=<¬ a₁>]
 1 -> 2 [label=<¬ a₂>]
 2 -> 3 [label=<b₂>]
 3 -> 4 [label=<¬ a₃>]
 4 -> 5 [label=<b₃>]
 6 -> 7 [label=<b₁>]
 7 -> 8 [label=<¬ b₂>]
 8 -> 9 [label=<a₂>]
 9 -> 10 [label=<¬ a₃>]
 10 -> 11 [label=<b₃>]

 0 [label=<{a₁,b₁}, {a₂,b₂}, {a₃,b₃}, {¬ a₁,¬ a₂}, {¬ a₂,¬ a₃}, {¬ a₁,¬ a₃}, {¬ b₁,¬ b₂}, {¬ b₂,¬ b₃}, {¬ b₁,¬ b₃}>]
 1 [label=<{a₂,b₂}, {a₃,b₃}, {¬ a₂}, {¬ a₃}, {¬ b₁,¬ b₂}, {¬ b₂,¬ b₃}, {¬ b₁,¬ b₃}>]
 2 [label=<{b₂}, {a₃,b₃}, {¬ a₃}, {¬ b₁,¬ b₂}, {¬ b₂,¬ b₃}, {¬ b₁,¬ b₃}>]
 3 [label=<{a₃,b₃}, {¬ a₃}, {¬ b₁}, {¬ b₃}>]
 4 [label=<{b₃}, {¬ b₁}, {¬ b₃}>]
 5 [label=<□>]
 6 [label=<{b₁}, {a₂,b₂}, {a₃,b₃}, {¬ a₂,¬ a₃}, {¬ b₁,¬ b₂}, {¬ b₂,¬ b₃}, {¬ b₁,¬ b₃}>]
 7 [label=<{a₂,b₂}, {a₃,b₃}, {¬ a₂,¬ a₃}, {¬ b₂}, {¬ b₃}>]
 8 [label=<{a₂}, {a₃,b₃}, {¬ a₂,¬ a₃}, {¬ b₃}>]
 9 [label=<{a₃,b₃}, {¬ a₃}, {¬ b₃}>]
 10 [label=<{b₃}, {¬ b₃}>]
 11 [label=<□>]
}
```

</center>